from django.contrib.auth.models import User
from django.db import models
from django.utils.translation import ugettext_lazy as _


# A very basic data model to begin with
# TODO: 
# - Create sensible default options with translation; 
# - Test the authentification framework; 
# - Ensure relevance of the comics building way : block by block;
# - Test upload to images file outside the DB ...;
# - ...
class Style(models.Model):                                                    
	"""
	TODO: The choice list should be in DB
	"""
	TAG_NAME_CHOICES = (
                (_('MG'), _('Manga')),
                (_('HF'), _('Heroic Fantasy')),
    )
	name = models.CharField(_('name'), max_length=30, choices=TAG_NAME_CHOICES)
	def __unicode__(self):
		return self.name
	
	class Meta:
		verbose_name = _('Style')
		verbose_name_plural = _('Styles')
		ordering = ('name',)

class UserProfile(models.Model):
	user = models.OneToOneField(User)	
	headshot = models.ImageField(upload_to='user_headshots')
	is_author = models.BooleanField()
	""" 
        The main difference is that the author have is own ads publisher
        The default behavior of class inheritance is to create OnetoOne relationship between parent and child 
        TODO: Which fields are required to interact with the ads publisher ?
        """
	ADS_PUBLISHER_CHOICES = (
    		('AS', 'Advert Stream'),
            ('TJ', 'Traffic Junky'),
    )
	ads_publisher = models.CharField(_('Ads publisher'), max_length=50, choices=ADS_PUBLISHER_CHOICES)
	ads_publisher_login = models.CharField(_('Ads publisher login'), max_length=50)

	class Meta:
		verbose_name = _('User profile')
		verbose_name_plural = _('Users profiles')

# TODO: See how to handle a group of authors and the revenue sharing ... later 

class Comic_block(models.Model):
	"""
	Let's view a comics as an images series
	"""
	name = models.CharField(_('name'), max_length=50)  # probably not useful, it's just simpler to assemble afterwards for author
	number = models.IntegerField(_('number'))
	content = models.ImageField(upload_to='block_contents')
	# TODO: probably not useful
	is_complete = models.BooleanField()
	upload_datetime = models.DateTimeField(_('upload_datetime'), auto_now=True)
	authors = models.ManyToManyField(UserProfile, verbose_name=_('authors'))
	def __unicode__(self):
		"""
		This idea is to return a visual identifier to the author, self.content is surely bogus to get the image path
		"""
		return self.name + ' ' + self.content

	class Meta:
		verbose_name = _('Comic_block')
		verbose_name_plural = _('Comic_blocks')
		ordering = ('name',)

class Comic(models.Model):
	""" 
	A comic is build from N blocks, whatever they are
	The ManytoMany relationship is not really required but it much more reflect really : authors can share block between comic
	"""
	title = models.CharField(_('title'), max_length=50)
	rating = models.FloatField()
	is_online = models.BooleanField()
	publication_date = models.DateField(auto_now=True)
	blocks = models.ManyToManyField(Comic_block)
	styles = models.ManyToManyField(Style)
	def __unicode__(self):
		return self.title
	
	class Meta:
		verbose_name = _('Comic')
		verbose_name_plural = _('Comics')
		ordering = ('title',)
